# AUTOGENERATED! DO NOT EDIT! File to edit: 03_main.ipynb (unless otherwise specified).

__all__ = ['COMMANDS', 'get_direction_idx', 'parse_args', 'Runner', 'run']

# Cell
from .direction import Direction
from .robot import *
from argparse import ArgumentParser
import re

# Cell
COMMANDS = ['MOVE', 'REPORT', 'LEFT', 'RIGHT']

# Cell
def get_direction_idx(direction, directions=['NORTH', 'EAST', 'SOUTH', 'WEST']):
    "returns the index of direction passed as input from `directions`."
    return directions.index(direction.upper())

# Cell
def parse_args():
    "argument parser to accept `path` as CLI arg."
    parser = ArgumentParser()
    parser.add_argument(
        '-path',
        '--cmd_file_path',
        required=True,
        help="absolute path to the `.txt` file containing commands for the ToyRobot."
    )
    parser.add_argument(
        '-v',
        '--verbose',
        help="Whether to print out every time a direction or pos change occurs.",
        action='store_true',
        default=False
    )
    args = parser.parse_args()
    return args

# Cell
class Runner:
    """
    This class takes care of the constraints for `ToyRobot` and also calls the required commands
    for the `ToyRobot` to do the actual moving, change direction and report.

    It can be considered to be a regular function with a state `self.robot`.

    It can be called just as a function passing in a list of args or nothing at all.
    If no args are passed, then the [ArgumentParser](https://docs.python.org/3/library/argparse.html#argparse.ArgumentParser)
    is run to get arguments from the command line. Otherwise, for testing some args can
    also be passed.

    This function reads the text file line by line, removes comma and whitespace and splits the
    string command to get actual arguments. Then based on these arguments the actual commands on
    `ToyRobot` are called.

    If 'PLACE' command is seen, then a new robot is created and assigned to self.robot if
    it is a valid 'PLACE' command.

    A valid command is one that places the `ToyRobot` inside the `Table`.

    Otherwise, if the command is not valid then no `ToyRobot` is instantiated and all other commands
    are ignored until a valid 'PLACE' command is seen.

    If there are two 'PLACE' commands inside the .txt file, then
    - both will run if they are both valid.
    - only valid command will run if one is valid.
    - no robot will be created if both are invalid.
    """

    def __init__(self):
        self.robot = None

    def __call__(self, args=None):
        "main function to run the package from command line."
        if not args:
            args = parse_args()
        # open text file from path passed as arg
        f = open(args.cmd_file_path, 'r')
        # read line by line and execute commands based on constraints in `problem.md`
        for line in f.readlines():
            # remove any extra whitespace and commas to get command
            cmds = re.findall(r'[^,\s]+', line)
            if cmds[0]=='PLACE':
                robot = ToyRobot.from_placement(
                    int(cmds[1]), int(cmds[2]), get_direction_idx(cmds[3]), args.verbose)
                if robot:
                    self.robot = robot
                    if args.verbose: self.robot.report()

            elif self.robot is not None and cmds[0] != 'PLACE' and cmds[0] in COMMANDS:
                cmd = f"self.robot.{cmds[0].lower()}()"
                exec(cmd)

# Cell
run = Runner()


# AUTOGENERATED! DO NOT EDIT! File to edit: 02_robot.ipynb (unless otherwise specified).

__all__ = ['ToyRobot', 'Table']

# Cell
from .direction import Direction
import numpy as np
from numpy import sin, cos, pi

# Cell
class ToyRobot:
    "The actual ToyRobot class the does the moving."
    def __init__(self, x, y, idx=None, th=5, tw=5, verbose=False):
        self.f = Direction(idx=idx)
        self.x = x
        self.y = y
        self.table = Table(th, tw)
        self.verbose = verbose

    def report(self):
        "report the current (X,Y) coordinates and direction."
        print(f"{self.x},{self.y},{self.f}")

    def left(self):
        "update direction anticlockwise."
        self.f = self.f.left()
        if self.verbose: self.report()

    def right(self):
        "update direction clockwise."
        self.f = self.f.right()
        if self.verbose: self.report()

    def move(self):
        """
        get curent direction, take a step in the direction and only update the X and Y coords
        if the new position is inside the table.
        """
        idx = self.f.get_idx()
        new_x = self.x + (sin(idx*(pi/2)).astype(int))
        new_y = self.y + (cos(idx*(pi/2)).astype(int))
        if (new_x, new_y) in self.table:
            self.x = new_x
            self.y = new_y
        if self.verbose: self.report()

    @classmethod
    def from_placement(cls, x:int, y:int, idx:int, verbose=False):
        "place a robot if it's inside the table and ignore otherwise"
        if not (isinstance(x, int) and isinstance(y, int)):
            raise ValueError(f"only integer values are acceptable for `x`, `y` coordinates.")

        if max(x,y)<=5:
            return cls(x, y, idx=idx, verbose=verbose)

# Cell
class Table:
    def __init__(self, h=5, w=5):
        self.h = h
        self.w = w

    def __contains__(self, o):
        if isinstance(o, ToyRobot):
            return min(self.h, self.w) >= max(o.x, o.y)
        else:
            return min(self.h, self.w) >= max(o[0], o[1])


# AUTOGENERATED! DO NOT EDIT! File to edit: 01_direction.ipynb (unless otherwise specified).

__all__ = ['Direction']

# Cell
class Direction:
    """
    A cyclic direction class to make rotation easier.

    The directions are written in a specific order of North, East, South, West
    instead of North, South, East, West to make rotation simpler.

    To rotate the direction clockwise, this class simply updates the `idx` to `idx+1`.
    To rotate the direction clockwise, this class simply updates the `idx` to `idx-1`.

    Returns:
        An instance of itself with the updated direction on calling either `left` or `right`.
    """
    def __init__(self, directions=['NORTH', 'EAST', 'SOUTH', 'WEST'], idx=None):
        # get random direction if not passed
        if idx is None:
            idx = np.random.choice(np.arange(4))
        self.directions = directions
        self.max_idx = len(directions)-1
        self.f = self.directions[idx]

    def get_idx(self):
        "return the current direction index from `self.directions`."
        return self.directions.index(self.f)

    def left(self):
        "first get current direction index, then subtract 1 to rotate left."
        idx = self.get_idx()
        new_idx = idx-1 if idx>=1 else self.max_idx
        return self.__class__(idx=new_idx)

    def right(self):
        "first get current direction index, then add 1 to rotate right."
        idx = self.get_idx()
        new_idx = idx+1 if idx<=2 else 0
        return self.__class__(idx=new_idx)

    def __repr__(self):
        return self.f


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Direction": "01_direction.ipynb",
         "ToyRobot": "02_robot.ipynb",
         "Table": "02_robot.ipynb",
         "COMMANDS": "03_main.ipynb",
         "get_direction_idx": "03_main.ipynb",
         "parse_args": "03_main.ipynb",
         "Runner": "03_main.ipynb",
         "run": "03_main.ipynb",
         "test_eq": "04_tests.ipynb",
         "test_stdout": "04_tests.ipynb"}

modules = ["direction.py",
           "robot.py",
           "main.py",
           "test.py"]

doc_url = "https://amaarora.github.io/toyrobot/"

git_url = "https://github.com/amaarora/toyrobot/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: 04_tests.ipynb (unless otherwise specified).

__all__ = ['test_eq', 'test_stdout']

# Cell
from .main import run
from io import StringIO
from contextlib import redirect_stdout

# Cell
def test_eq(a, b):
    "test a equals b"
    assert a==b, f"test failed, {a} not equal to {b}"

# Cell
def test_stdout(f, args, exp):
    "test that `f` print out `exp` to stdout."
    s = StringIO()
    with redirect_stdout(s):
        f(args)
    test_eq(s.getvalue(), f'{exp}\n' if len(exp)>0 else '')